@@ -15,8 +15,7 @@
 
 import logging
 
-from openstack import exceptions as sdk_exceptions
-from openstack.network.v2 import floating_ip as _floating_ip
+from osc_lib.command import command
 from osc_lib import utils
 
 from openstackclient.i18n import _
@@ -66,6 +65,10 @@ def _get_attrs(client_manager, parsed_args):
     if parsed_args.fixed_ip_address:
         attrs['fixed_ip_address'] = parsed_args.fixed_ip_address
 
+    if parsed_args.qos_policy:
+        attrs['qos_policy_id'] = network_client.find_qos_policy(
+            parsed_args.qos_policy, ignore_missing=False).id
+
     if parsed_args.description is not None:
         attrs['description'] = parsed_args.description
 
@@ -81,58 +84,6 @@ def _get_attrs(client_manager, parsed_args):
     return attrs
 
 
-def _find_floating_ip(
-    session,
-    ip_cache,
-    name_or_id,
-    ignore_missing=True,
-    **params
-):
-    """Find a floating IP by IP or ID
-
-    The SDK's find_ip() can only locate a floating IP by ID so we have
-    to do this ourselves.
-    """
-
-    def _get_one_match(name_or_id):
-        """Given a list of results, return the match"""
-        the_result = None
-        for maybe_result in ip_cache:
-            id_value = maybe_result.id
-            ip_value = maybe_result.floating_ip_address
-
-            if (id_value == name_or_id) or (ip_value == name_or_id):
-                # Only allow one resource to be found. If we already
-                # found a match, raise an exception to show it.
-                if the_result is None:
-                    the_result = maybe_result
-                else:
-                    msg = "More than one %s exists with the name '%s'."
-                    msg = (msg % (_floating_ip.FloatingIP, name_or_id))
-                    raise sdk_exceptions.DuplicateResource(msg)
-
-        return the_result
-
-    # Try to short-circuit by looking directly for a matching ID.
-    try:
-        match = _floating_ip.FloatingIP.existing(id=name_or_id, **params)
-        return (match.get(session), ip_cache)
-    except sdk_exceptions.NotFoundException:
-        pass
-
-    if len(ip_cache) == 0:
-        ip_cache = list(_floating_ip.FloatingIP.list(session, **params))
-
-    result = _get_one_match(name_or_id)
-    if result is not None:
-        return (result, ip_cache)
-
-    if ignore_missing:
-        return (None, ip_cache)
-    raise sdk_exceptions.ResourceNotFound(
-        "No %s found for %s" % (_floating_ip.FloatingIP.__name__, name_or_id))
-
-
 class CreateFloatingIP(common.NetworkAndComputeShowOne):
     _description = _("Create floating IP")
 
@@ -162,16 +113,21 @@ class CreateFloatingIP(common.NetworkAndComputeShowOne):
         )
         parser.add_argument(
             '--floating-ip-address',
-            metavar='<floating-ip-address>',
+            metavar='<ip-address>',
             dest='floating_ip_address',
             help=_("Floating IP address")
         )
         parser.add_argument(
             '--fixed-ip-address',
-            metavar='<fixed-ip-address>',
+            metavar='<ip-address>',
             dest='fixed_ip_address',
             help=_("Fixed IP address mapped to the floating IP")
         )
+        parser.add_argument(
+            '--qos-policy',
+            metavar='<qos-policy>',
+            help=_("Attach QoS policy to the floating IP (name or ID)")
+        )
         parser.add_argument(
             '--description',
             metavar='<description>',
@@ -193,9 +149,9 @@ class CreateFloatingIP(common.NetworkAndComputeShowOne):
         return (display_columns, data)
 
     def take_action_compute(self, client, parsed_args):
-        obj = client.floating_ips.create(parsed_args.network)
-        columns = _get_columns(obj._info)
-        data = utils.get_dict_properties(obj._info, columns)
+        obj = client.api.floating_ip_create(parsed_args.network)
+        columns = _get_columns(obj)
+        data = utils.get_dict_properties(obj, columns)
         return (columns, data)
 
 
@@ -240,27 +196,14 @@ class DeleteFloatingIP(common.NetworkAndComputeDelete):
         return parser
 
     def take_action_network(self, client, parsed_args):
-        (obj, self.ip_cache) = _find_floating_ip(
-            self.app.client_manager.sdk_connection.session,
-            self.ip_cache,
+        obj = client.find_ip(
             self.r,
             ignore_missing=False,
         )
         client.delete_ip(obj)
 
     def take_action_compute(self, client, parsed_args):
-        obj = utils.find_resource(client.floating_ips, self.r)
-        client.floating_ips.delete(obj.id)
-
-    def take_action(self, parsed_args):
-        """Implements a naive cache for the list of floating IPs"""
-
-        # NOTE(dtroyer): This really only prevents multiple list()
-        #                calls when performing multiple resource deletes
-        #                in a single command. In an interactive session
-        #                each delete command will call list().
-        self.ip_cache = []
-        super(DeleteFloatingIP, self).take_action(parsed_args)
+        client.api.floating_ip_delete(self.r)
 
 
 class DeleteIPFloating(DeleteFloatingIP):
@@ -307,7 +250,7 @@ class ListFloatingIP(common.NetworkAndComputeLister):
         )
         parser.add_argument(
             '--fixed-ip-address',
-            metavar='<fixed-ip-address>',
+            metavar='<ip-address>',
             help=_("List floating IP(s) according to "
                    "given fixed IP address")
         )
@@ -423,10 +366,10 @@ class ListFloatingIP(common.NetworkAndComputeLister):
             'Pool',
         )
 
-        data = client.floating_ips.list()
+        data = client.api.floating_ip_list()
 
         return (headers,
-                (utils.get_item_properties(
+                (utils.get_dict_properties(
                     s, columns,
                     formatters={},
                 ) for s in data))
@@ -456,12 +399,66 @@ class ListIPFloating(ListFloatingIP):
             client, parsed_args)
 
 
+class SetFloatingIP(command.Command):
+    _description = _("Set floating IP Properties")
+
+    def get_parser(self, prog_name):
+        parser = super(SetFloatingIP, self).get_parser(prog_name)
+        parser.add_argument(
+            'floating_ip',
+            metavar='<floating-ip>',
+            help=_("Floating IP to associate (IP address or ID)"))
+        parser.add_argument(
+            '--port',
+            metavar='<port>',
+            required=True,
+            help=_("Associate the floating IP with port (name or ID)")),
+        parser.add_argument(
+            '--fixed-ip-address',
+            metavar='<ip-address>',
+            dest='fixed_ip_address',
+            help=_("Fixed IP of the port "
+                   "(required only if port has multiple IPs)")
+        )
+        qos_policy_group = parser.add_mutually_exclusive_group()
+        qos_policy_group.add_argument(
+            '--qos-policy',
+            metavar='<qos-policy>',
+            help=_("Attach QoS policy to the floating IP (name or ID)")
+        )
+        qos_policy_group.add_argument(
+            '--no-qos-policy',
+            action='store_true',
+            help=_("Remove the QoS policy attached to the floating IP")
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        client = self.app.client_manager.network
+        attrs = {}
+        obj = client.find_ip(
+            parsed_args.floating_ip,
+            ignore_missing=False,
+        )
+        port = client.find_port(parsed_args.port,
+                                ignore_missing=False)
+        attrs['port_id'] = port.id
+        if parsed_args.fixed_ip_address:
+            attrs['fixed_ip_address'] = parsed_args.fixed_ip_address
+
+        if parsed_args.qos_policy:
+            attrs['qos_policy_id'] = client.find_qos_policy(
+                parsed_args.qos_policy, ignore_missing=False).id
+
+        if 'no_qos_policy' in parsed_args and parsed_args.no_qos_policy:
+            attrs['qos_policy_id'] = None
+
+        client.update_ip(obj, **attrs)
+
+
 class ShowFloatingIP(common.NetworkAndComputeShowOne):
     _description = _("Display floating IP details")
 
-    # ip_cache is unused here but is a side effect of _find_floating_ip()
-    ip_cache = []
-
     def update_parser_common(self, parser):
         parser.add_argument(
             'floating_ip',
@@ -471,9 +468,7 @@ class ShowFloatingIP(common.NetworkAndComputeShowOne):
         return parser
 
     def take_action_network(self, client, parsed_args):
-        (obj, self.ip_cache) = _find_floating_ip(
-            self.app.client_manager.sdk_connection.session,
-            [],
+        obj = client.find_ip(
             parsed_args.floating_ip,
             ignore_missing=False,
         )
@@ -482,12 +477,9 @@ class ShowFloatingIP(common.NetworkAndComputeShowOne):
         return (display_columns, data)
 
     def take_action_compute(self, client, parsed_args):
-        obj = utils.find_resource(
-            client.floating_ips,
-            parsed_args.floating_ip,
-        )
-        columns = _get_columns(obj._info)
-        data = utils.get_dict_properties(obj._info, columns)
+        obj = client.api.floating_ip_find(parsed_args.floating_ip)
+        columns = _get_columns(obj)
+        data = utils.get_dict_properties(obj, columns)
         return (columns, data)
 
 
@@ -513,3 +505,42 @@ class ShowIPFloating(ShowFloatingIP):
                            'Please use "floating ip show" instead.'))
         return super(ShowIPFloating, self).take_action_compute(
             client, parsed_args)
+
+
+class UnsetFloatingIP(command.Command):
+    _description = _("Unset floating IP Properties")
+
+    def get_parser(self, prog_name):
+        parser = super(UnsetFloatingIP, self).get_parser(prog_name)
+        parser.add_argument(
+            'floating_ip',
+            metavar='<floating-ip>',
+            help=_("Floating IP to disassociate (IP address or ID)"))
+        parser.add_argument(
+            '--port',
+            action='store_true',
+            default=False,
+            help=_("Disassociate any port associated with the floating IP")
+        )
+        parser.add_argument(
+            '--qos-policy',
+            action='store_true',
+            default=False,
+            help=_("Remove the QoS policy attached to the floating IP")
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        client = self.app.client_manager.network
+        obj = client.find_ip(
+            parsed_args.floating_ip,
+            ignore_missing=False,
+        )
+        attrs = {}
+        if parsed_args.port:
+            attrs['port_id'] = None
+        if parsed_args.qos_policy:
+            attrs['qos_policy_id'] = None
+
+        if attrs:
+            client.update_ip(obj, **attrs)
