@@ -21,6 +21,7 @@ import six
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
 from openstackclient.network import common
+from openstackclient.network import sdk_utils
 from openstackclient.network import utils as network_utils
 
 
@@ -34,6 +35,7 @@ def _format_network_security_group_rules(sg_rules):
             sg_rule.pop(key)
         sg_rule.pop('security_group_id', None)
         sg_rule.pop('tenant_id', None)
+        sg_rule.pop('project_id', None)
     return utils.format_list_of_dicts(sg_rules)
 
 
@@ -72,29 +74,15 @@ _formatters_compute = {
 
 
 def _get_columns(item):
-    # Build the display columns and a list of the property columns
-    # that need to be mapped (display column name, property name).
-    columns = list(item.keys())
-    property_column_mappings = []
-    if 'security_group_rules' in columns:
-        columns.append('rules')
-        columns.remove('security_group_rules')
-        property_column_mappings.append(('rules', 'security_group_rules'))
-    if 'tenant_id' in columns:
-        columns.remove('tenant_id')
-        if 'project_id' not in columns:
-            columns.append('project_id')
-        property_column_mappings.append(('project_id', 'tenant_id'))
-    display_columns = sorted(columns)
-
-    # Build the property columns and apply any column mappings.
-    property_columns = sorted(columns)
-    for property_column_mapping in property_column_mappings:
-        property_index = property_columns.index(property_column_mapping[0])
-        property_columns[property_index] = property_column_mapping[1]
-    return tuple(display_columns), property_columns
+    column_map = {
+        'security_group_rules': 'rules',
+        'tenant_id': 'project_id',
+    }
+    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)
 
 
+# TODO(abhiraut): Use the SDK resource mapped attribute names once the
+# OSC minimum requirements include SDK 1.0.
 class CreateSecurityGroup(common.NetworkAndComputeShowOne):
     _description = _("Create a new security group")
 
@@ -152,13 +140,13 @@ class CreateSecurityGroup(common.NetworkAndComputeShowOne):
 
     def take_action_compute(self, client, parsed_args):
         description = self._get_description(parsed_args)
-        obj = client.security_groups.create(
+        obj = client.api.security_group_create(
             parsed_args.name,
             description,
         )
-        display_columns, property_columns = _get_columns(obj._info)
+        display_columns, property_columns = _get_columns(obj)
         data = utils.get_dict_properties(
-            obj._info,
+            obj,
             property_columns,
             formatters=_formatters_compute
         )
@@ -186,10 +174,11 @@ class DeleteSecurityGroup(common.NetworkAndComputeDelete):
         client.delete_security_group(obj)
 
     def take_action_compute(self, client, parsed_args):
-        data = utils.find_resource(client.security_groups, self.r)
-        client.security_groups.delete(data.id)
+        client.api.security_group_delete(self.r)
 
 
+# TODO(rauta): Use the SDK resource mapped attribute names once
+# the OSC minimum requirements include SDK 1.0.
 class ListSecurityGroup(common.NetworkAndComputeLister):
     _description = _("List security groups")
 
@@ -220,21 +209,6 @@ class ListSecurityGroup(common.NetworkAndComputeLister):
         )
         return parser
 
-    def _get_return_data(self, data, include_project=True):
-        columns = (
-            "ID",
-            "Name",
-            "Description",
-        )
-        column_headers = columns
-        if include_project:
-            columns = columns + ('Tenant ID',)
-            column_headers = column_headers + ('Project',)
-        return (column_headers,
-                (utils.get_item_properties(
-                    s, columns,
-                ) for s in data))
-
     def take_action_network(self, client, parsed_args):
         filters = {}
         if parsed_args.project:
@@ -245,13 +219,46 @@ class ListSecurityGroup(common.NetworkAndComputeLister):
                 parsed_args.project_domain,
             ).id
             filters['tenant_id'] = project_id
-        return self._get_return_data(client.security_groups(**filters))
+            filters['project_id'] = project_id
+        data = client.security_groups(**filters)
+
+        columns = (
+            "ID",
+            "Name",
+            "Description",
+            "Project ID"
+        )
+        column_headers = (
+            "ID",
+            "Name",
+            "Description",
+            "Project"
+        )
+        return (column_headers,
+                (utils.get_item_properties(
+                    s, columns,
+                ) for s in data))
 
     def take_action_compute(self, client, parsed_args):
         search = {'all_tenants': parsed_args.all_projects}
-        data = client.security_groups.list(search_opts=search)
-        return self._get_return_data(data,
-                                     include_project=parsed_args.all_projects)
+        data = client.api.security_group_list(
+            # TODO(dtroyer): add limit, marker
+            search_opts=search,
+        )
+
+        columns = (
+            "ID",
+            "Name",
+            "Description",
+        )
+        column_headers = columns
+        if parsed_args.all_projects:
+            columns = columns + ('Tenant ID',)
+            column_headers = column_headers + ('Project',)
+        return (column_headers,
+                (utils.get_dict_properties(
+                    s, columns,
+                ) for s in data))
 
 
 class SetSecurityGroup(common.NetworkAndComputeCommand):
@@ -289,23 +296,20 @@ class SetSecurityGroup(common.NetworkAndComputeCommand):
         client.update_security_group(obj, **attrs)
 
     def take_action_compute(self, client, parsed_args):
-        data = utils.find_resource(
-            client.security_groups,
-            parsed_args.group,
-        )
+        data = client.api.security_group_find(parsed_args.group)
 
         if parsed_args.name is not None:
-            data.name = parsed_args.name
+            data['name'] = parsed_args.name
         if parsed_args.description is not None:
-            data.description = parsed_args.description
+            data['description'] = parsed_args.description
 
         # NOTE(rtheis): Previous behavior did not raise a CommandError
         # if there were no updates. Maintain this behavior and issue
         # the update.
-        client.security_groups.update(
+        client.api.security_group_set(
             data,
-            data.name,
-            data.description,
+            data['name'],
+            data['description'],
         )
 
 
@@ -332,13 +336,10 @@ class ShowSecurityGroup(common.NetworkAndComputeShowOne):
         return (display_columns, data)
 
     def take_action_compute(self, client, parsed_args):
-        obj = utils.find_resource(
-            client.security_groups,
-            parsed_args.group,
-        )
-        display_columns, property_columns = _get_columns(obj._info)
+        obj = client.api.security_group_find(parsed_args.group)
+        display_columns, property_columns = _get_columns(obj)
         data = utils.get_dict_properties(
-            obj._info,
+            obj,
             property_columns,
             formatters=_formatters_compute
         )
