@@ -13,24 +13,37 @@
 
 """Subnet pool action implementations"""
 
-from openstackclient.common import command
-from openstackclient.common import exceptions
-from openstackclient.common import parseractions
-from openstackclient.common import utils
+import copy
+import logging
+
+from osc_lib.cli import parseractions
+from osc_lib.command import command
+from osc_lib import exceptions
+from osc_lib import utils
+
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
+from openstackclient.network import sdk_utils
+from openstackclient.network.v2 import _tag
+
+
+LOG = logging.getLogger(__name__)
 
 
 def _get_columns(item):
-    columns = list(item.keys())
-    if 'tenant_id' in columns:
-        columns.remove('tenant_id')
-        columns.append('project_id')
-    return tuple(sorted(columns))
+    column_map = {
+        'default_prefix_length': 'default_prefixlen',
+        'is_shared': 'shared',
+        'maximum_prefix_length': 'max_prefixlen',
+        'minimum_prefix_length': 'min_prefixlen',
+        'tenant_id': 'project_id',
+    }
+    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)
 
 
 _formatters = {
     'prefixes': utils.format_list,
+    'tags': utils.format_list,
 }
 
 
@@ -75,6 +88,12 @@ def _get_attrs(client_manager, parsed_args):
         ).id
         attrs['tenant_id'] = project_id
 
+    if parsed_args.description is not None:
+        attrs['description'] = parsed_args.description
+
+    if parsed_args.default_quota is not None:
+        attrs['default_quota'] = int(parsed_args.default_quota)
+
     return attrs
 
 
@@ -125,8 +144,10 @@ def _add_default_options(parser):
     )
 
 
+# TODO(rtheis): Use the SDK resource mapped attribute names once the
+# OSC minimum requirements include SDK 1.0.
 class CreateSubnetPool(command.ShowOne):
-    """Create subnet pool"""
+    _description = _("Create subnet pool")
 
     def get_parser(self, prog_name):
         parser = super(CreateSubnetPool, self).get_parser(prog_name)
@@ -161,6 +182,18 @@ class CreateSubnetPool(command.ShowOne):
             action='store_true',
             help=_("Set this subnet pool as not shared"),
         )
+        parser.add_argument(
+            '--description',
+            metavar='<description>',
+            help=_("Set subnet pool description")
+        )
+        parser.add_argument(
+            '--default-quota',
+            type=int,
+            metavar='<num-ip-addresses>',
+            help=_("Set default quota for subnet pool as the number of"
+                   "IP addresses allowed in a subnet")),
+        _tag.add_tag_option_to_parser_for_create(parser, _('subnet pool'))
         return parser
 
     def take_action(self, parsed_args):
@@ -170,31 +203,51 @@ class CreateSubnetPool(command.ShowOne):
         if "prefixes" not in attrs:
             attrs['prefixes'] = []
         obj = client.create_subnet_pool(**attrs)
-        columns = _get_columns(obj)
+        # tags cannot be set when created, so tags need to be set later.
+        _tag.update_tags_for_set(client, obj, parsed_args)
+        display_columns, columns = _get_columns(obj)
         data = utils.get_item_properties(obj, columns, formatters=_formatters)
-        return (columns, data)
+        return (display_columns, data)
 
 
 class DeleteSubnetPool(command.Command):
-    """Delete subnet pool"""
+    _description = _("Delete subnet pool(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteSubnetPool, self).get_parser(prog_name)
         parser.add_argument(
             'subnet_pool',
             metavar='<subnet-pool>',
-            help=_("Subnet pool to delete (name or ID)")
+            nargs='+',
+            help=_("Subnet pool(s) to delete (name or ID)")
         )
         return parser
 
     def take_action(self, parsed_args):
         client = self.app.client_manager.network
-        obj = client.find_subnet_pool(parsed_args.subnet_pool)
-        client.delete_subnet_pool(obj)
+        result = 0
+
+        for pool in parsed_args.subnet_pool:
+            try:
+                obj = client.find_subnet_pool(pool, ignore_missing=False)
+                client.delete_subnet_pool(obj)
+            except Exception as e:
+                result += 1
+                LOG.error(_("Failed to delete subnet pool with "
+                          "name or ID '%(pool)s': %(e)s"),
+                          {'pool': pool, 'e': e})
+
+        if result > 0:
+            total = len(parsed_args.subnet_pool)
+            msg = (_("%(result)s of %(total)s subnet pools failed "
+                   "to delete.") % {'result': result, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
+# TODO(rtheis): Use only the SDK resource mapped attribute names once the
+# OSC minimum requirements include SDK 1.0.
 class ListSubnetPool(command.Lister):
-    """List subnet pools"""
+    _description = _("List subnet pools")
 
     def get_parser(self, prog_name):
         parser = super(ListSubnetPool, self).get_parser(prog_name)
@@ -204,41 +257,89 @@ class ListSubnetPool(command.Lister):
             default=False,
             help=_("List additional fields in output")
         )
+        shared_group = parser.add_mutually_exclusive_group()
+        shared_group.add_argument(
+            '--share',
+            action='store_true',
+            help=_("List subnet pools shared between projects"),
+        )
+        shared_group.add_argument(
+            '--no-share',
+            action='store_true',
+            help=_("List subnet pools not shared between projects"),
+        )
+        default_group = parser.add_mutually_exclusive_group()
+        default_group.add_argument(
+            '--default',
+            action='store_true',
+            help=_("List subnet pools used as the default external "
+                   "subnet pool"),
+        )
+        default_group.add_argument(
+            '--no-default',
+            action='store_true',
+            help=_("List subnet pools not used as the default external "
+                   "subnet pool")
+        )
+        parser.add_argument(
+            '--project',
+            metavar='<project>',
+            help=_("List subnet pools according to their project (name or ID)")
+        )
+        identity_common.add_project_domain_option_to_parser(parser)
+        parser.add_argument(
+            '--name',
+            metavar='<name>',
+            help=_("List only subnet pools of given name in output")
+        )
+        parser.add_argument(
+            '--address-scope',
+            metavar='<address-scope>',
+            help=_("List only subnet pools of given address scope "
+                   "in output (name or ID)")
+        )
+        _tag.add_tag_filtering_option_to_parser(parser, _('subnet pools'))
         return parser
 
     def take_action(self, parsed_args):
-        data = self.app.client_manager.network.subnet_pools()
+        identity_client = self.app.client_manager.identity
+        network_client = self.app.client_manager.network
+        filters = {}
+        if parsed_args.share:
+            filters['shared'] = True
+            filters['is_shared'] = True
+        elif parsed_args.no_share:
+            filters['shared'] = False
+            filters['is_shared'] = False
+        if parsed_args.default:
+            filters['is_default'] = True
+        elif parsed_args.no_default:
+            filters['is_default'] = False
+        if parsed_args.project:
+            project_id = identity_common.find_project(
+                identity_client,
+                parsed_args.project,
+                parsed_args.project_domain,
+            ).id
+            filters['tenant_id'] = project_id
+            filters['project_id'] = project_id
+        if parsed_args.name is not None:
+            filters['name'] = parsed_args.name
+        if parsed_args.address_scope:
+            address_scope = network_client.find_address_scope(
+                parsed_args.address_scope,
+                ignore_missing=False)
+            filters['address_scope_id'] = address_scope.id
+        _tag.get_tag_filtering_args(parsed_args, filters)
+        data = network_client.subnet_pools(**filters)
 
+        headers = ('ID', 'Name', 'Prefixes')
+        columns = ('id', 'name', 'prefixes')
         if parsed_args.long:
-            headers = (
-                'ID',
-                'Name',
-                'Prefixes',
-                'Default Prefix Length',
-                'Address Scope',
-                'Default Subnet Pool',
-                'Shared',
-            )
-            columns = (
-                'id',
-                'name',
-                'prefixes',
-                'default_prefixlen',
-                'address_scope_id',
-                'is_default',
-                'shared',
-            )
-        else:
-            headers = (
-                'ID',
-                'Name',
-                'Prefixes',
-            )
-            columns = (
-                'id',
-                'name',
-                'prefixes',
-            )
+            headers += ('Default Prefix Length', 'Address Scope',
+                        'Default Subnet Pool', 'Shared', 'Tags')
+            columns += ('default_prefix_length', 'address_scope_id',
+                        'is_default', 'is_shared', 'tags')
 
         return (headers,
                 (utils.get_item_properties(
@@ -247,8 +348,10 @@ class ListSubnetPool(command.Lister):
                 ) for s in data))
 
 
+# TODO(rtheis): Use the SDK resource mapped attribute names once the
+# OSC minimum requirements include SDK 1.0.
 class SetSubnetPool(command.Command):
-    """Set subnet pool properties"""
+    _description = _("Set subnet pool properties")
 
     def get_parser(self, prog_name):
         parser = super(SetSubnetPool, self).get_parser(prog_name)
@@ -277,6 +380,18 @@ class SetSubnetPool(command.Command):
             help=_("Remove address scope associated with the subnet pool")
         )
         _add_default_options(parser)
+        parser.add_argument(
+            '--description',
+            metavar='<description>',
+            help=_("Set subnet pool description")
+        )
+        parser.add_argument(
+            '--default-quota',
+            type=int,
+            metavar='<num-ip-addresses>',
+            help=_("Set default quota for subnet pool as the number of"
+                   "IP addresses allowed in a subnet")),
+        _tag.add_tag_option_to_parser_for_set(parser, _('subnet pool'))
 
         return parser
 
@@ -286,19 +401,19 @@ class SetSubnetPool(command.Command):
                                       ignore_missing=False)
 
         attrs = _get_attrs(self.app.client_manager, parsed_args)
-        if attrs == {}:
-            msg = _("Nothing specified to be set")
-            raise exceptions.CommandError(msg)
 
         # Existing prefixes must be a subset of the new prefixes.
         if 'prefixes' in attrs:
             attrs['prefixes'].extend(obj.prefixes)
 
-        client.update_subnet_pool(obj, **attrs)
+        if attrs:
+            client.update_subnet_pool(obj, **attrs)
+        # tags is a subresource and it needs to be updated separately.
+        _tag.update_tags_for_set(client, obj, parsed_args)
 
 
 class ShowSubnetPool(command.ShowOne):
-    """Display subnet pool details"""
+    _description = _("Display subnet pool details")
 
     def get_parser(self, prog_name):
         parser = super(ShowSubnetPool, self).get_parser(prog_name)
@@ -315,6 +430,49 @@ class ShowSubnetPool(command.ShowOne):
             parsed_args.subnet_pool,
             ignore_missing=False
         )
-        columns = _get_columns(obj)
+        display_columns, columns = _get_columns(obj)
         data = utils.get_item_properties(obj, columns, formatters=_formatters)
-        return (columns, data)
+        return (display_columns, data)
+
+
+class UnsetSubnetPool(command.Command):
+    _description = _("Unset subnet pool properties")
+
+    def get_parser(self, prog_name):
+        parser = super(UnsetSubnetPool, self).get_parser(prog_name)
+        parser.add_argument(
+            '--pool-prefix',
+            metavar='<pool-prefix>',
+            action='append',
+            dest='prefixes',
+            help=_('Remove subnet pool prefixes (in CIDR notation). '
+                   '(repeat option to unset multiple prefixes).'),
+        )
+        parser.add_argument(
+            'subnet_pool',
+            metavar="<subnet-pool>",
+            help=_("Subnet pool to modify (name or ID)")
+        )
+        _tag.add_tag_option_to_parser_for_unset(parser, _('subnet pool'))
+        return parser
+
+    def take_action(self, parsed_args):
+        client = self.app.client_manager.network
+        obj = client.find_subnet_pool(
+            parsed_args.subnet_pool, ignore_missing=False)
+        tmp_prefixes = copy.deepcopy(obj.prefixes)
+        attrs = {}
+        if parsed_args.prefixes:
+            for prefix in parsed_args.prefixes:
+                try:
+                    tmp_prefixes.remove(prefix)
+                except ValueError:
+                    msg = _(
+                        "Subnet pool does not "
+                        "contain prefix %s") % prefix
+                    raise exceptions.CommandError(msg)
+            attrs['prefixes'] = tmp_prefixes
+        if attrs:
+            client.update_subnet_pool(obj, **attrs)
+        # tags is a subresource and it needs to be updated separately.
+        _tag.update_tags_for_unset(client, obj, parsed_args)
