@@ -14,6 +14,7 @@
 #
 
 import copy
+import datetime
 import uuid
 
 from keystoneauth1 import access
@@ -34,6 +35,7 @@ DOMAIN = {
     'name': domain_name,
     'description': domain_description,
     'enabled': True,
+    'tags': [],
     'links': base_url + 'domains/' + domain_id,
 }
 
@@ -115,6 +117,7 @@ PROJECT = {
     'description': project_description,
     'enabled': True,
     'domain_id': domain_id,
+    'tags': [],
     'links': base_url + 'projects/' + project_id,
 }
 
@@ -124,6 +127,7 @@ PROJECT_2 = {
     'description': project_description + 'plus four more',
     'enabled': True,
     'domain_id': domain_id,
+    'tags': [],
     'links': base_url + 'projects/' + project_id,
 }
 
@@ -145,6 +149,7 @@ PROJECT_WITH_PARENT = {
     'enabled': True,
     'domain_id': domain_id,
     'parent_id': project_id,
+    'tags': [],
     'links': base_url + 'projects/' + (project_id + '-with-parent'),
 }
 
@@ -155,6 +160,7 @@ PROJECT_WITH_GRANDPARENT = {
     'enabled': True,
     'domain_id': domain_id,
     'parent_id': PROJECT_WITH_PARENT['id'],
+    'tags': [],
     'links': base_url + 'projects/' + (project_id + '-with-grandparent'),
 }
 
@@ -223,6 +229,20 @@ ENDPOINT = {
     'links': base_url + 'endpoints/' + endpoint_id,
 }
 
+endpoint_group_id = 'eg-123'
+endpoint_group_description = 'eg 123 description'
+endpoint_group_filters = {
+    'service_id': service_id,
+    'region_id': endpoint_region,
+}
+
+ENDPOINT_GROUP = {
+    'id': endpoint_group_id,
+    'filters': endpoint_group_filters,
+    'description': endpoint_group_description,
+    'links': base_url + 'endpoint_groups/' + endpoint_group_id,
+}
+
 user_id = 'bbbbbbb-aaaa-aaaa-aaaa-bbbbbbbaaaa'
 user_name = 'paul'
 user_description = 'Sir Paul'
@@ -438,6 +458,34 @@ OAUTH_VERIFIER = {
     'oauth_verifier': oauth_verifier_pin
 }
 
+app_cred_id = 'app-cred-id'
+app_cred_name = 'testing_app_cred'
+app_cred_role = {"id": role_id, "name": role_name, "domain": None},
+app_cred_description = 'app credential for testing'
+app_cred_expires = datetime.datetime(2022, 1, 1, 0, 0)
+app_cred_expires_str = app_cred_expires.strftime('%Y-%m-%dT%H:%M:%S%z')
+app_cred_secret = 'moresecuresecret'
+APP_CRED_BASIC = {
+    'id': app_cred_id,
+    'name': app_cred_name,
+    'project_id': project_id,
+    'roles': app_cred_role,
+    'description': None,
+    'expires_at': None,
+    'unrestricted': False,
+    'secret': app_cred_secret
+}
+APP_CRED_OPTIONS = {
+    'id': app_cred_id,
+    'name': app_cred_name,
+    'project_id': project_id,
+    'roles': app_cred_role,
+    'description': app_cred_description,
+    'expires_at': app_cred_expires_str,
+    'unrestricted': False,
+    'secret': app_cred_secret
+}
+
 
 def fake_auth_ref(fake_token, fake_service=None):
     """Create an auth_ref using keystoneauth's fixtures"""
@@ -495,6 +543,8 @@ class FakeIdentityv3Client(object):
         self.endpoints.resource_class = fakes.FakeResource(None, {})
         self.endpoint_filter = mock.Mock()
         self.endpoint_filter.resource_class = fakes.FakeResource(None, {})
+        self.endpoint_groups = mock.Mock()
+        self.endpoint_groups.resource_class = fakes.FakeResource(None, {})
         self.groups = mock.Mock()
         self.groups.resource_class = fakes.FakeResource(None, {})
         self.oauth1 = mock.Mock()
@@ -523,6 +573,9 @@ class FakeIdentityv3Client(object):
         self.auth = FakeAuth()
         self.auth.client = mock.Mock()
         self.auth.client.resource_class = fakes.FakeResource(None, {})
+        self.application_credentials = mock.Mock()
+        self.application_credentials.resource_class = fakes.FakeResource(None,
+                                                                         {})
 
 
 class FakeFederationManager(object):
@@ -619,6 +672,7 @@ class FakeProject(object):
             'is_domain': False,
             'domain_id': 'domain-id-' + uuid.uuid4().hex,
             'parent_id': 'parent-id-' + uuid.uuid4().hex,
+            'tags': [],
             'links': 'links-' + uuid.uuid4().hex,
         }
         project_info.update(attrs)
@@ -666,6 +720,7 @@ class FakeDomain(object):
             'name': 'domain-name-' + uuid.uuid4().hex,
             'description': 'domain-description-' + uuid.uuid4().hex,
             'enabled': True,
+            'tags': [],
             'links': 'links-' + uuid.uuid4().hex,
         }
         domain_info.update(attrs)
