@@ -11,10 +11,10 @@
 #   under the License.
 #
 
-import mock
-from mock import call
 import random
 
+import mock
+from mock import call
 from osc_lib import exceptions
 from osc_lib import utils
 
@@ -64,6 +64,7 @@ class TestCreateNetworkIdentityV3(TestNetwork):
         'ipv4_address_scope',
         'ipv6_address_scope',
         'is_default',
+        'mtu',
         'name',
         'port_security_enabled',
         'project_id',
@@ -87,6 +88,7 @@ class TestCreateNetworkIdentityV3(TestNetwork):
         _network.ipv4_address_scope_id,
         _network.ipv6_address_scope_id,
         _network.is_default,
+        _network.mtu,
         _network.name,
         _network.is_port_security_enabled,
         _network.project_id,
@@ -149,6 +151,7 @@ class TestCreateNetworkIdentityV3(TestNetwork):
             "--disable",
             "--share",
             "--description", self._network.description,
+            "--mtu", self._network.mtu,
             "--project", self.project.name,
             "--project-domain", self.domain.name,
             "--availability-zone-hint", "nova",
@@ -165,6 +168,7 @@ class TestCreateNetworkIdentityV3(TestNetwork):
             ('disable', True),
             ('share', True),
             ('description', self._network.description),
+            ('mtu', self._network.mtu),
             ('project', self.project.name),
             ('project_domain', self.domain.name),
             ('availability_zone_hints', ["nova"]),
@@ -188,6 +192,7 @@ class TestCreateNetworkIdentityV3(TestNetwork):
             'name': self._network.name,
             'shared': True,
             'description': self._network.description,
+            'mtu': self._network.mtu,
             # TODO(dtroyer): Remove tenant_id when we clean up the SDK refactor
             'tenant_id': self.project.id,
             'project_id': self.project.id,
@@ -286,6 +291,7 @@ class TestCreateNetworkIdentityV2(TestNetwork):
         'ipv4_address_scope',
         'ipv6_address_scope',
         'is_default',
+        'mtu',
         'name',
         'port_security_enabled',
         'project_id',
@@ -309,6 +315,7 @@ class TestCreateNetworkIdentityV2(TestNetwork):
         _network.ipv4_address_scope_id,
         _network.ipv6_address_scope_id,
         _network.is_default,
+        _network.mtu,
         _network.name,
         _network.is_port_security_enabled,
         _network.project_id,
@@ -1023,6 +1030,7 @@ class TestShowNetwork(TestNetwork):
         'ipv4_address_scope',
         'ipv6_address_scope',
         'is_default',
+        'mtu',
         'name',
         'port_security_enabled',
         'project_id',
@@ -1046,6 +1054,7 @@ class TestShowNetwork(TestNetwork):
         _network.ipv4_address_scope_id,
         _network.ipv6_address_scope_id,
         _network.is_default,
+        _network.mtu,
         _network.name,
         _network.is_port_security_enabled,
         _network.project_id,
